@@ -15,6 +15,7 @@ import (
 )
 
 type currentPitr struct {
+	slicer *pitr.Slicer
 	wakeup chan struct{} // to wake up a slicer on demand (not to wait for the tick)
 	cancel context.CancelFunc
 }
@@ -36,30 +37,10 @@ func (a *Agent) unsetPitr() {
 	a.mx.Unlock()
 }
 
-func (a *Agent) ispitr() bool {
+func (a *Agent) getPitr() *currentPitr {
 	a.mx.Lock()
 	defer a.mx.Unlock()
-	return a.pitrjob != nil
-}
-
-func (a *Agent) cancelPitr() {
-	a.mx.Lock()
-	defer a.mx.Unlock()
-	if a.pitrjob == nil {
-		return
-	}
-
-	a.pitrjob.cancel()
-}
-
-func (a *Agent) wakeupPitr() {
-	a.mx.Lock()
-	defer a.mx.Unlock()
-	if a.pitrjob == nil {
-		return
-	}
-
-	a.pitrjob.wakeup <- struct{}{}
+	return a.pitrjob
 }
 
 const pitrCheckPeriod = time.Second * 15
@@ -91,17 +72,46 @@ func (a *Agent) pitr() (err error) {
 		return nil
 	}
 
-	on, err := a.pbm.IsPITR()
-	if err != nil {
-		return errors.Wrap(err, "check if on")
+	cfg, err := a.pbm.GetConfig()
+	if err != nil && !errors.Is(err, mongo.ErrNoDocuments) {
+		return errors.Wrap(err, "get conf")
 	}
-	if !on {
-		a.cancelPitr()
+
+	p := a.getPitr()
+
+	if !cfg.PITR.Enabled {
+		if p != nil {
+			p.cancel()
+		}
 		return nil
 	}
 
-	// we already do the job
-	if on && a.ispitr() {
+	ep, err := a.pbm.GetEpoch()
+	if err != nil {
+		return errors.Wrap(err, "get epoch")
+	}
+
+	l := a.log.NewEvent(string(pbm.CmdPITR), "", "", ep.TS())
+
+	spant := time.Duration(cfg.PITR.OplogSpanMin * float64(time.Minute))
+	if spant == 0 {
+		spant = pbm.PITRdefaultSpan
+	}
+
+	// already do the job
+	if p != nil {
+		// update slicer span
+		cspan := p.slicer.GetSpan()
+		if p.slicer != nil && cspan != spant {
+			l.Debug("set pitr span to %v", spant)
+			p.slicer.SetSpan(spant)
+
+			// wake up slicer only if span became smaller
+			if spant < cspan {
+				a.pitrjob.wakeup <- struct{}{}
+			}
+		}
+
 		return nil
 	}
 
@@ -135,19 +145,6 @@ func (a *Agent) pitr() (err error) {
 		return nil
 	}
 
-	ibcp := pitr.NewBackup(a.node.RS(), a.pbm, a.node)
-
-	err = ibcp.Catchup()
-	if err != nil {
-		return errors.Wrap(err, "defining starting point for the backup")
-	}
-
-	ep, err := a.pbm.GetEpoch()
-	if err != nil {
-		return errors.Wrap(err, "get epoch")
-	}
-	l := a.log.NewEvent(string(pbm.CmdPITR), "", "", ep.TS())
-
 	stg, err := a.pbm.GetStorage(l)
 	if err != nil {
 		return errors.Wrap(err, "unable to get storage configuration")
@@ -170,16 +167,28 @@ func (a *Agent) pitr() (err error) {
 		return nil
 	}
 
+	ibcp := pitr.NewSlicer(a.node.RS(), a.pbm, a.node, stg, ep)
+	ibcp.SetSpan(spant)
+
+	err = ibcp.Catchup()
+	if err != nil {
+		if err := lock.Release(); err != nil {
+			l.Error("release lock: %v", err)
+		}
+		return errors.Wrap(err, "catchup")
+	}
+
 	go func() {
 		ctx, cancel := context.WithCancel(context.Background())
 		w := make(chan struct{})
 
 		a.setPitr(&currentPitr{
+			slicer: ibcp,
 			cancel: cancel,
 			wakeup: w,
 		})
 
-		streamErr := ibcp.Stream(ctx, ep, w, stg, pbm.CompressionTypeS2)
+		streamErr := ibcp.Stream(ctx, w, pbm.CompressionTypeS2)
 		if streamErr != nil {
 			switch streamErr.(type) {
 			case pitr.ErrOpMoved:
@@ -228,7 +237,7 @@ func (a *Agent) pitrLockCheck() (moveOn bool, err error) {
 
 // PITRestore starts the point-in-time recovery
 func (a *Agent) PITRestore(r pbm.PITRestoreCmd, opid pbm.OPID, ep pbm.Epoch) {
-	l := a.log.NewEvent(string(pbm.CmdPITR), time.Unix(r.TS, 0).UTC().Format(time.RFC3339), opid.String(), ep.TS())
+	l := a.log.NewEvent(string(pbm.CmdPITRestore), time.Unix(r.TS, 0).UTC().Format(time.RFC3339), opid.String(), ep.TS())
 
 	nodeInfo, err := a.node.GetInfo()
 	if err != nil {
