@@ -17,6 +17,7 @@ func (c *Cluster) PITRbasic() {
 	bcpName := c.Backup()
 
 	c.pitrOn()
+	log.Println("turn on PITR")
 	defer c.pitrOff()
 
 	rand.Seed(time.Now().UnixNano())
@@ -40,10 +41,20 @@ func (c *Cluster) PITRbasic() {
 
 	c.BackupWaitDone(bcpName)
 
+	c.printBcpList()
+
+	time.Sleep(time.Second * 30)
+	log.Printf("Sleep for %v", time.Second*30)
+
+	bcp2 := c.Backup()
+	c.BackupWaitDone(bcp2)
+
 	ds := time.Second * 30 * time.Duration(rand.Int63n(5)+2)
 	log.Printf("Generating data for %v", ds)
 	time.Sleep(ds)
 
+	c.printBcpList()
+
 	log.Println("Get reference time")
 	var lastt primitive.Timestamp
 	for name, c := range counters {
@@ -68,6 +79,14 @@ func (c *Cluster) PITRbasic() {
 		c.bcheckClear(name, shard)
 	}
 
+	log.Printf("Deleting backup %v", bcp2)
+	err := c.mongopbm.DeleteBackup(bcp2)
+	if err != nil {
+		log.Fatalf("Error: delete backup %s: %v", bcp2, err)
+	}
+
+	c.printBcpList()
+
 	// +1 sec since we are PITR restore done up to < time (not <=)
 	c.PITRestore(time.Unix(int64(lastt.T), 0).Add(time.Second * 1))
 
