@@ -8,10 +8,10 @@ import (
 	"log"
 	"time"
 
-	"github.com/mongodb/mongo-tools-common/db"
-	mlog "github.com/mongodb/mongo-tools-common/log"
-	"github.com/mongodb/mongo-tools-common/options"
-	"github.com/mongodb/mongo-tools-common/progress"
+	"github.com/mongodb/mongo-tools/common/db"
+	mlog "github.com/mongodb/mongo-tools/common/log"
+	"github.com/mongodb/mongo-tools/common/options"
+	"github.com/mongodb/mongo-tools/common/progress"
 	"github.com/mongodb/mongo-tools/mongodump"
 	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson/primitive"
@@ -143,9 +143,9 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 			return
 		}
 
-		err = b.cn.SetBalancerStatus(pbm.BalancerModeOn)
-		if err != nil {
-			l.Error("set balancer ON: %v", err)
+		errd := b.cn.SetBalancerStatus(pbm.BalancerModeOn)
+		if errd != nil {
+			l.Error("set balancer ON: %v", errd)
 			return
 		}
 		l.Debug("set balancer on")
@@ -186,7 +186,14 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 		return errors.Wrap(err, "waiting for start")
 	}
 
+	oplog := NewOplog(b.node)
+	oplogTS, err := oplog.LastWrite()
+	if err != nil {
+		return errors.Wrap(err, "define oplog start position")
+	}
+
 	rsMeta.Status = pbm.StatusRunning
+	rsMeta.FirstWriteTS = oplogTS
 	err = b.cn.AddRSMeta(bcp.Name, rsMeta)
 	if err != nil {
 		return errors.Wrap(err, "add shard's metadata")
@@ -200,6 +207,11 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 			}
 			return errors.Wrap(err, "check cluster for backup started")
 		}
+
+		err = b.setClusterFirstWrite(bcp.Name)
+		if err != nil {
+			return errors.Wrap(err, "set cluster last write ts")
+		}
 	}
 
 	// Waiting for cluster's StatusRunning to move further.
@@ -208,17 +220,6 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 		return errors.Wrap(err, "waiting for running")
 	}
 
-	oplog := NewOplog(b.node)
-	oplogTS, err := oplog.LastWrite()
-	if err != nil {
-		return errors.Wrap(err, "define oplog start position")
-	}
-
-	err = b.cn.SetRSFirstWrite(bcp.Name, rsMeta.Name, oplogTS)
-	if err != nil {
-		return errors.Wrap(err, "set shard's first write ts")
-	}
-
 	// Save users and roles to the tmp collections so the restore would copy that data
 	// to the system collections. Have to do this because of issues with the restore and preserverUUID.
 	// see: https://jira.percona.com/browse/PBM-636 and comments
@@ -255,7 +256,10 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 		sz *= 4
 	}
 
-	dump := newDump(b.node.ConnURI(), b.node.DumpConns())
+	dump, err := newDump(b.node.ConnURI(), b.node.DumpConns())
+	if err != nil {
+		return errors.Wrap(err, "init mongodump options")
+	}
 	_, err = Upload(ctx, dump, stg, bcp.Compression, rsMeta.DumpName, sz)
 	if err != nil {
 		return errors.Wrap(err, "mongodump")
@@ -283,9 +287,9 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 			return errors.Wrap(err, "check cluster for dump done")
 		}
 
-		err = b.setClusterFirtsLastWrite(bcp.Name)
+		err = b.setClusterLastWrite(bcp.Name)
 		if err != nil {
-			return errors.Wrap(err, "set cluster first & last write ts")
+			return errors.Wrap(err, "set cluster last write ts")
 		}
 	}
 
@@ -312,6 +316,13 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 	}
 
 	if inf.IsLeader() {
+		epch, err := b.cn.ResetEpoch()
+		if err != nil {
+			l.Error("reset epoch")
+		} else {
+			l.Debug("epoch set to %v", epch)
+		}
+
 		err = b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusDone, inf, nil)
 		if err != nil {
 			return errors.Wrap(err, "check cluster for backup done")
@@ -626,24 +637,37 @@ func writeMeta(stg storage.Storage, meta *pbm.BackupMeta) error {
 	return errors.Wrap(err, "write to store")
 }
 
-func (b *Backup) setClusterFirtsLastWrite(bcpName string) error {
+func (b *Backup) setClusterFirstWrite(bcpName string) error {
 	bmeta, err := b.cn.GetBackupMeta(bcpName)
 	if err != nil {
 		return errors.Wrap(err, "get backup metadata")
 	}
 
-	var fw, lw primitive.Timestamp
+	var fw primitive.Timestamp
 	for _, rs := range bmeta.Replsets {
-		if primitive.CompareTimestamp(lw, rs.LastWriteTS) == -1 {
-			lw = rs.LastWriteTS
-		}
-
 		if fw.T == 0 || primitive.CompareTimestamp(fw, rs.FirstWriteTS) == 1 {
 			fw = rs.FirstWriteTS
 		}
 	}
 
-	err = b.cn.SetFirstLastWrite(bcpName, fw, lw)
+	err = b.cn.SetFirstWrite(bcpName, fw)
+	return errors.Wrap(err, "set timestamp")
+}
+
+func (b *Backup) setClusterLastWrite(bcpName string) error {
+	bmeta, err := b.cn.GetBackupMeta(bcpName)
+	if err != nil {
+		return errors.Wrap(err, "get backup metadata")
+	}
+
+	var lw primitive.Timestamp
+	for _, rs := range bmeta.Replsets {
+		if primitive.CompareTimestamp(lw, rs.LastWriteTS) == -1 {
+			lw = rs.LastWriteTS
+		}
+	}
+
+	err = b.cn.SetLastWrite(bcpName, lw)
 	return errors.Wrap(err, "set timestamp")
 }
 
@@ -652,22 +676,30 @@ type mdump struct {
 	conns int
 }
 
-func newDump(curi string, conns int) *mdump {
+func newDump(curi string, conns int) (*mdump, error) {
 	if conns <= 0 {
 		conns = 1
 	}
-	return &mdump{
-		opts: &options.ToolOptions{
-			AppName:    "mongodump",
-			VersionStr: "0.0.1",
-			URI:        &options.URI{ConnectionString: curi},
-			Auth:       &options.Auth{},
-			Namespace:  &options.Namespace{},
-			Connection: &options.Connection{},
-			Direct:     true,
-		},
-		conns: conns,
+
+	var err error
+
+	opts := options.New("mongodump", "0.0.1", "none", "", true, options.EnabledOptions{Auth: true, Connection: true, Namespace: true, URI: true})
+	opts.URI, err = options.NewURI(curi)
+	if err != nil {
+		return nil, errors.Wrap(err, "parse connection string")
 	}
+
+	err = opts.NormalizeOptionsAndURI()
+	if err != nil {
+		return nil, errors.Wrap(err, "parse opts")
+	}
+
+	opts.Direct = true
+
+	return &mdump{
+		opts:  opts,
+		conns: conns,
+	}, nil
 }
 
 // "logger" for the mongodup's ProgressManager.
