@@ -37,6 +37,7 @@ type NodeInfo struct {
 	IsPrimary                    bool                 `bson:"ismaster"`
 	Secondary                    bool                 `bson:"secondary,omitempty"`
 	Hidden                       bool                 `bson:"hidden,omitempty"`
+	Passive                      bool                 `bson:"passive,omitempty"`
 	ConfigSvr                    int                  `bson:"configsvr,omitempty"`
 	Me                           string               `bson:"me"`
 	LastWrite                    MongoLastWrite       `bson:"lastWrite"`
@@ -55,6 +56,12 @@ func (i *NodeInfo) IsLeader() bool {
 	return !i.IsSharded() || i.ReplsetRole() == ReplRoleConfigSrv
 }
 
+// IsClusterLeader - cluster leader is a primary node on configsrv
+// or just primary node in non-sharded replicaset
+func (i *NodeInfo) IsClusterLeader() bool {
+	return i.IsPrimary && i.Me == i.Primary && i.IsLeader()
+}
+
 // ReplsetRole returns replset role in sharded clister
 func (i *NodeInfo) ReplsetRole() ReplRole {
 	switch {
@@ -184,3 +191,21 @@ type AuthUserRoles struct {
 	Role string `bson:"role" json:"role"`
 	DB   string `bson:"db" json:"db"`
 }
+
+type BalancerMode string
+
+const (
+	BalancerModeOn  BalancerMode = "full"
+	BalancerModeOff BalancerMode = "off"
+)
+
+type BalancerStatus struct {
+	Mode              BalancerMode `bson:"mode" json:"mode"`
+	InBalancerRound   bool         `bson:"inBalancerRound" json:"inBalancerRound"`
+	NumBalancerRounds int64        `bson:"numBalancerRounds" json:"numBalancerRounds"`
+	Ok                int          `bson:"ok" json:"ok"`
+}
+
+func (b *BalancerStatus) IsOn() bool {
+	return b.Mode == BalancerModeOn
+}
