@@ -3,6 +3,7 @@ package pbm
 import (
 	"fmt"
 	"path"
+	"sort"
 	"strconv"
 	"strings"
 	"time"
@@ -99,35 +100,33 @@ func (p *PBM) pitrChunk(rs string, sort int) (*PITRChunk, error) {
 	return chnk, errors.Wrap(err, "decode")
 }
 
-// PITRGetChunkContains returns a pitr slice chunk that belongs to the
-// given replica set and contains the given timestamp
-func (p *PBM) PITRGetChunkContains(rs string, ts primitive.Timestamp) (*PITRChunk, error) {
-	res := p.Conn.Database(DB).Collection(PITRChunksCollection).FindOne(
-		p.ctx,
-		bson.D{
-			{"rs", rs},
-			{"start_ts", bson.M{"$lte": ts}},
-			{"end_ts", bson.M{"$gte": ts}},
-		},
-	)
-	if res.Err() != nil {
-		return nil, errors.Wrap(res.Err(), "get")
-	}
-
-	chnk := new(PITRChunk)
-	err := res.Decode(chnk)
-	return chnk, errors.Wrap(err, "decode")
-}
-
 // PITRGetChunksSlice returns slice of PITR oplog chunks which Start TS
-// lies in a given time frame
+// lies in a given time frame. Returns all chunks if `to` is 0.
 func (p *PBM) PITRGetChunksSlice(rs string, from, to primitive.Timestamp) ([]PITRChunk, error) {
 	q := bson.D{}
 	if rs != "" {
 		q = bson.D{{"rs", rs}}
 	}
-	q = append(q, bson.E{"start_ts", bson.M{"$gte": from, "$lte": to}})
+	if to.T > 0 {
+		q = append(q, bson.E{"start_ts", bson.M{"$gte": from, "$lte": to}})
+	}
 
+	return p.pitrGetChunksSlice(rs, q)
+}
+
+// PITRGetChunksSliceUntil returns slice of PITR oplog chunks that starts up until timestamp (exclusively)
+func (p *PBM) PITRGetChunksSliceUntil(rs string, t primitive.Timestamp) ([]PITRChunk, error) {
+	q := bson.D{}
+	if rs != "" {
+		q = bson.D{{"rs", rs}}
+	}
+
+	q = append(q, bson.E{"start_ts", bson.M{"$lt": t}})
+
+	return p.pitrGetChunksSlice(rs, q)
+}
+
+func (p *PBM) pitrGetChunksSlice(rs string, q bson.D) ([]PITRChunk, error) {
 	cur, err := p.Conn.Database(DB).Collection(PITRChunksCollection).Find(
 		p.ctx,
 		q,
@@ -140,7 +139,6 @@ func (p *PBM) PITRGetChunksSlice(rs string, from, to primitive.Timestamp) ([]PIT
 	defer cur.Close(p.ctx)
 
 	chnks := []PITRChunk{}
-
 	for cur.Next(p.ctx) {
 		var chnk PITRChunk
 		err := cur.Decode(&chnk)
@@ -181,9 +179,9 @@ func (p *PBM) PITRAddChunk(c PITRChunk) error {
 }
 
 type Timeline struct {
-	Start uint32
-	End   uint32
-	Size  int64
+	Start uint32 `json:"start"`
+	End   uint32 `json:"end"`
+	Size  int64  `json:"-"`
 }
 
 const tlTimeFormat = "2006-01-02T15:04:05"
@@ -200,7 +198,7 @@ func (t Timeline) String() string {
 // any saved chunk already belongs to some valid timeline,
 // the slice wouldn't be done otherwise.
 // `flist` is a cache of chunk sizes.
-func (p *PBM) PITRGetValidTimelines(rs string, until int64, flist map[string]int64) (tlines []Timeline, err error) {
+func (p *PBM) PITRGetValidTimelines(rs string, until primitive.Timestamp, flist map[string]int64) (tlines []Timeline, err error) {
 	fch, err := p.PITRFirstChunkMeta(rs)
 	if err != nil {
 		return nil, errors.Wrap(err, "get the oldest chunk")
@@ -209,7 +207,7 @@ func (p *PBM) PITRGetValidTimelines(rs string, until int64, flist map[string]int
 		return nil, nil
 	}
 
-	slices, err := p.PITRGetChunksSlice(rs, fch.StartTS, primitive.Timestamp{T: uint32(until), I: 0})
+	slices, err := p.PITRGetChunksSlice(rs, fch.StartTS, until)
 	if err != nil {
 		return nil, errors.Wrap(err, "get slice")
 	}
@@ -223,18 +221,44 @@ func (p *PBM) PITRGetValidTimelines(rs string, until int64, flist map[string]int
 	return gettimelines(slices), nil
 }
 
+// PITRTimelines returns cluster-wide time ranges valid for PITR restore
+func (p *PBM) PITRTimelines() (tlines []Timeline, err error) {
+	shards, err := p.ClusterMembers(nil)
+	if err != nil {
+		return nil, errors.Wrap(err, "get cluster members")
+	}
+
+	now, err := p.ClusterTime()
+	if err != nil {
+		return nil, errors.Wrap(err, "get cluster time")
+	}
+
+	var tlns [][]Timeline
+	for _, s := range shards {
+		t, err := p.PITRGetValidTimelines(s.RS, now, nil)
+		if err != nil {
+			return nil, errors.Wrapf(err, "get PITR timelines for %s replset", s.RS)
+		}
+		if len(t) != 0 {
+			tlns = append(tlns, t)
+		}
+	}
+
+	return MergeTimelines(tlns...), nil
+}
+
 func gettimelines(slices []PITRChunk) (tlines []Timeline) {
 	var tl Timeline
-	var prevEnd uint32
+	var prevEnd primitive.Timestamp
 	for _, s := range slices {
-		if prevEnd != 0 && prevEnd != s.StartTS.T {
+		if prevEnd.T != 0 && primitive.CompareTimestamp(prevEnd, s.StartTS) == -1 {
 			tlines = append(tlines, tl)
 			tl = Timeline{}
 		}
 		if tl.Start == 0 {
 			tl.Start = s.StartTS.T
 		}
-		prevEnd = s.EndTS.T
+		prevEnd = s.EndTS
 		tl.End = s.EndTS.T
 		tl.Size += s.size
 	}
@@ -244,14 +268,22 @@ func gettimelines(slices []PITRChunk) (tlines []Timeline) {
 	return tlines
 }
 
-type tlineMerge struct {
-	tl    Timeline
-	repls int
+type gap struct {
+	s, e uint32
 }
 
+type gaps []gap
+
+func (x gaps) Len() int { return len(x) }
+func (x gaps) Less(i, j int) bool {
+	return x[i].s < x[j].s || (x[i].s == x[j].s && x[i].e < x[j].e)
+}
+func (x gaps) Swap(i, j int) { x[i], x[j] = x[j], x[i] }
+
 // MergeTimelines merges overlapping sets on timelines
-// it preresumes timelines already sorted and don't start from 0
+// it presumes timelines are sorted and don't start from 0
 func MergeTimelines(tlns ...[]Timeline) []Timeline {
+	// fast paths
 	if len(tlns) == 0 {
 		return nil
 	}
@@ -259,59 +291,101 @@ func MergeTimelines(tlns ...[]Timeline) []Timeline {
 		return tlns[0]
 	}
 
-	// defining the base timeline set
-	// (set with the less anount of timelines)
-	ln := len(tlns[0])
-	ri := 0
-	for i, tl := range tlns {
-		if len(tl) < ln {
-			ln = len(tl)
-			ri = i
+	// First, we define the avaliagble range. It equals to the beginning of the latest start of the first
+	// timeline of any set and to the earliest end of the last timeline of any set. Then define timelines' gaps
+	// merge overlapping and apply resulted gap on the avaliagble range.
+	//
+	// given timelines:
+	// 1 2 3 4     7 8 10 11          16 17 18 19 20
+	//     3 4 5 6 7 8 10 11 12    15 16 17
+	// 1 2 3 4 5 6 7 8 10 11 12       16 17 18
+	//
+	// aavliable range:
+	//     3 4 5 6 7 8 10 11 12 13 15 16 17
+	// merged gaps:
+	//         5 6           12 13 15       18 19 20
+	// result:
+	//     3 4     7 8 10 11          16 17
+	//
+
+	// limits of the avaliagble range
+	// `start` is the lates start the timelines range
+	// `end` - is the earliest end
+	var start, end uint32
+
+	// iterating through the timelines  1) define `start` and `end`,
+	// 2) defiene gaps and add them into slice.
+	var g gaps
+	for _, tln := range tlns {
+		if len(tln) == 0 {
+			continue
 		}
-	}
-	if ri != 0 {
-		tlns[0], tlns[ri] = tlns[ri], tlns[0]
-	}
 
-	rtl := make([]tlineMerge, ln)
-	for i, v := range tlns[0] {
-		rtl[i] = tlineMerge{tl: v, repls: 1}
-	}
+		if tln[0].Start > start {
+			start = tln[0].Start
+		}
 
-	// for each timeilne in the base set we're looking
-	// for the overlaping timeline in the rest of the sets
-	for i := range rtl {
-	RSLOOP:
-		for j := 1; j < len(tlns); j++ {
-			tl := tlns[j]
+		if end == 0 || tln[len(tln)-1].End < end {
+			end = tln[len(tln)-1].End
+		}
 
-			for _, t := range tl {
-				if rtl[i].tl.End <= t.Start || t.End <= rtl[i].tl.Start {
-					continue
-				}
-				if t.Start > rtl[i].tl.Start {
-					rtl[i].tl.Start = t.Start
-				}
-
-				if t.End < rtl[i].tl.End {
-					rtl[i].tl.End = t.End
-				}
-
-				rtl[i].repls++
-				continue RSLOOP
+		if len(tln) == 1 {
+			continue
+		}
+		var ls uint32
+		for i, t := range tln {
+			if i == 0 {
+				ls = t.End
+				continue
 			}
+			g = append(g, gap{ls, t.Start})
+			ls = t.End
 		}
+	}
+	sort.Sort(g)
 
+	// if no gaps, just return avaliable range
+	if len(g) == 0 {
+		return []Timeline{{Start: start, End: end}}
 	}
 
-	tlines := []Timeline{}
-	for _, v := range rtl {
-		if v.repls == len(tlns) {
-			tlines = append(tlines, v.tl)
+	// merge overlapping gaps
+	var g2 gaps
+	var cend uint32
+	for _, gp := range g {
+		if gp.e <= start {
+			continue
+		}
+		if gp.s >= end {
+			break
+		}
+
+		if len(g2) > 0 {
+			cend = g2[len(g2)-1].e
+		}
+
+		if gp.s > cend {
+			g2 = append(g2, gp)
+			continue
+		}
+		if gp.e > cend {
+			g2[len(g2)-1].e = gp.e
 		}
 	}
 
-	return tlines
+	// split avaliable timeline with gaps
+	var ret []Timeline
+	for _, g := range g2 {
+		if start < g.s {
+			ret = append(ret, Timeline{Start: start, End: g.s})
+		}
+		start = g.e
+	}
+	if start < end {
+		ret = append(ret, Timeline{Start: start, End: end})
+	}
+
+	return ret
 }
 
 // PITRmetaFromFName parses given file name and returns PITRChunk metadata
