@@ -3,17 +3,18 @@ package restore
 import (
 	"context"
 	"encoding/json"
+	"io"
 	"strings"
 	"time"
 
-	"github.com/mongodb/mongo-tools-common/db"
-	mlog "github.com/mongodb/mongo-tools-common/log"
-	"github.com/mongodb/mongo-tools-common/options"
+	mlog "github.com/mongodb/mongo-tools/common/log"
+	"github.com/mongodb/mongo-tools/common/options"
 	"github.com/mongodb/mongo-tools/mongorestore"
 	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
+	"go.mongodb.org/mongo-driver/mongo/writeconcern"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
 	"github.com/percona/percona-backup-mongodb/pbm/log"
@@ -52,7 +53,9 @@ var excludeFromRestore = []string{
 	"config.system.sessions",
 	"config.cache.*",
 	"config.shards",
+	"config.transactions",
 	"admin.system.version",
+	"config.system.indexBuilds",
 }
 
 type Restore struct {
@@ -143,7 +146,7 @@ func (r *Restore) PITR(cmd pbm.PITRestoreCmd, opid pbm.OPID, l *log.Event) (err
 		return err
 	}
 
-	err = r.PreparePITR(cmd.TS)
+	err = r.PreparePITR(cmd.TS, cmd.Bcp)
 	if err != nil {
 		return err
 	}
@@ -222,12 +225,15 @@ func (r *Restore) init(name string, opid pbm.OPID, l *log.Event) (err error) {
 		return errors.Wrap(err, "define mongo version")
 	}
 
-	r.oplog = NewOplog(r.node, mgoV, preserveUUID)
+	r.oplog, err = NewOplog(r.node, mgoV, preserveUUID)
+	if err != nil {
+		return errors.Wrap(err, "create oplog")
+	}
 
 	return nil
 }
 
-func (r *Restore) PreparePITR(ts int64) (err error) {
+func (r *Restore) PreparePITR(ts int64, bcp string) (err error) {
 	r.pitrLastTS = ts
 
 	if r.nodeInfo.IsLeader() {
@@ -239,20 +245,14 @@ func (r *Restore) PreparePITR(ts int64) (err error) {
 
 	pts := primitive.Timestamp{T: uint32(ts), I: 0}
 
-	lastChunk, err := r.cn.PITRGetChunkContains(r.nodeInfo.SetName, pts)
+	err = r.setBcp(bcp, pts)
 	if err != nil {
-		return errors.Wrap(err, "define last oplog slice")
+		return errors.Wrap(err, "set snapshot")
 	}
 
-	r.bcp, err = r.cn.GetLastBackup(&lastChunk.EndTS)
-	if err != nil {
-		return errors.Wrap(err, "define last backup")
-	}
-	if r.bcp == nil {
-		return errors.Errorf("no backup found before ts %v", lastChunk.EndTS)
-	}
+	r.log.Info("snapshot %s", r.bcp.Name)
 
-	err = r.prepareChunks(r.bcp.LastWriteTS, lastChunk.StartTS)
+	err = r.prepareChunks(pts)
 	if err != nil {
 		return errors.Wrap(err, "verify oplog slices chain")
 	}
@@ -260,20 +260,52 @@ func (r *Restore) PreparePITR(ts int64) (err error) {
 	return r.prepareSnapshot()
 }
 
-// prepareChunks ensures integrity of oplog slices (timeline is continuous)
+func (r *Restore) setBcp(bcp string, targetTS primitive.Timestamp) (err error) {
+	if bcp != "" {
+		r.bcp, err = r.cn.GetBackupMeta(bcp)
+		if err != nil {
+			return errors.Wrap(err, "get backup")
+		}
+		if primitive.CompareTimestamp(r.bcp.LastWriteTS, targetTS) >= 0 {
+			return errors.Wrap(err, "snapshot's last write is later than the target time. Try to set an earlier snapshot. Or leave the snapshot empty so PBM will choose one.")
+		}
+
+		return nil
+	}
+
+	r.bcp, err = r.cn.GetLastBackup(&targetTS)
+	if err != nil {
+		return errors.Wrap(err, "define last backup")
+	}
+	if r.bcp == nil {
+		return errors.Errorf("no backup found before ts %v", targetTS)
+	}
+
+	return nil
+}
+
+// prepareChunks ensures integrity of oplog slices (timeline is contiguous - there are no gaps)
 // and chunks exists on the storage
-func (r *Restore) prepareChunks(from, to primitive.Timestamp) error {
-	chunks, err := r.cn.PITRGetChunksSlice(r.nodeInfo.SetName, from, to)
+func (r *Restore) prepareChunks(target primitive.Timestamp) error {
+	chunks, err := r.cn.PITRGetChunksSlice(r.nodeInfo.SetName, r.bcp.LastWriteTS, target)
 	if err != nil {
 		return errors.Wrap(err, "get chunks index")
 	}
 
-	nextStart := from
+	if len(chunks) == 0 {
+		return errors.New("no chunks found")
+	}
+
+	if primitive.CompareTimestamp(chunks[len(chunks)-1].EndTS, target) == -1 {
+		return errors.Errorf("no chunk with the target time, the last chunk ends on %v", chunks[len(chunks)-1].EndTS)
+	}
+
+	last := r.bcp.LastWriteTS
 	for _, c := range chunks {
-		if primitive.CompareTimestamp(nextStart, c.StartTS) != 0 {
-			return errors.Errorf("integrity vilolated, expect chunk with start_ts %v, but got %v", nextStart, c.StartTS)
+		if primitive.CompareTimestamp(last, c.StartTS) == -1 {
+			return errors.Errorf("integrity vilolated, expect chunk with start_ts %v, but got %v", last, c.StartTS)
 		}
-		nextStart = c.EndTS
+		last = c.EndTS
 
 		_, err := r.stg.FileStat(c.FName)
 		if err != nil {
@@ -427,64 +459,10 @@ func (r *Restore) RunSnapshot() (err error) {
 	}
 	defer dumpReader.Close()
 
-	topts := options.ToolOptions{
-		AppName:    "mongodump",
-		VersionStr: "0.0.1",
-		URI:        &options.URI{ConnectionString: r.node.ConnURI()},
-		Auth:       &options.Auth{},
-		Namespace:  &options.Namespace{},
-		Connection: &options.Connection{},
-		Direct:     true,
-	}
-
-	rsession, err := db.NewSessionProvider(topts)
+	// Restore snapshot (mongorestore)
+	err = r.snapshot(dumpReader)
 	if err != nil {
-		return errors.Wrap(err, "create session for the dump restore")
-	}
-
-	cfg, err := r.cn.GetConfig()
-	if err != nil {
-		return errors.Wrap(err, "unable to get PBM config settings")
-	}
-
-	batchSize := batchSizeDefault
-	if cfg.Restore.BatchSize > 0 {
-		batchSize = cfg.Restore.BatchSize
-	}
-	numInsertionWorkers := numInsertionWorkersDefault
-	if cfg.Restore.NumInsertionWorkers > 0 {
-		numInsertionWorkers = cfg.Restore.NumInsertionWorkers
-	}
-
-	mr := mongorestore.MongoRestore{
-		SessionProvider: rsession,
-		ToolOptions:     &topts,
-		// Have to handle users and roles on our own.
-		// see: https://jira.percona.com/browse/PBM-636 and comments.
-		SkipUsersAndRoles: true,
-		InputOptions: &mongorestore.InputOptions{
-			Archive: "-",
-		},
-		OutputOptions: &mongorestore.OutputOptions{
-			BulkBufferSize:           batchSize,
-			BypassDocumentValidation: true,
-			Drop:                     true,
-			NumInsertionWorkers:      numInsertionWorkers,
-			NumParallelCollections:   1,
-			PreserveUUID:             preserveUUID,
-			StopOnError:              true,
-			WriteConcern:             "majority",
-		},
-		NSOptions: &mongorestore.NSOptions{
-			NSExclude: excludeFromRestore,
-		},
-		InputReader: dumpReader,
-	}
-
-	rdumpResult := mr.Restore()
-	mr.Close()
-	if rdumpResult.Err != nil {
-		return errors.Wrapf(rdumpResult.Err, "restore mongo dump (successes: %d / fails: %d)", rdumpResult.Successes, rdumpResult.Failures)
+		return errors.Wrap(err, "mongorestore")
 	}
 
 	err = r.cn.ChangeRestoreRSState(r.name, r.nodeInfo.SetName, pbm.StatusDumpDone, "")
@@ -517,9 +495,9 @@ func (r *Restore) RunSnapshot() (err error) {
 	}
 
 	defer func() {
-		err = pbm.DropTMPcoll(r.cn.Context(), r.node.Session())
-		if err != nil {
-			r.log.Warning("drop tmp collections: %v", err)
+		errd := pbm.DropTMPcoll(r.cn.Context(), r.node.Session())
+		if errd != nil {
+			r.log.Warning("drop tmp collections: %v", errd)
 		}
 	}()
 
@@ -546,6 +524,70 @@ func (r *Restore) RunSnapshot() (err error) {
 	return nil
 }
 
+func (r *Restore) snapshot(input io.Reader) (err error) {
+	topts := options.New("mongorestore", "0.0.1", "none", "", true, options.EnabledOptions{Auth: true, Connection: true, Namespace: true, URI: true})
+	topts.URI, err = options.NewURI(r.node.ConnURI())
+	if err != nil {
+		return errors.Wrap(err, "parse connection string")
+	}
+
+	err = topts.NormalizeOptionsAndURI()
+	if err != nil {
+		return errors.Wrap(err, "parse opts")
+	}
+
+	topts.Direct = true
+	topts.WriteConcern = writeconcern.New(writeconcern.WMajority())
+
+	cfg, err := r.cn.GetConfig()
+	if err != nil {
+		return errors.Wrap(err, "unable to get PBM config settings")
+	}
+
+	batchSize := batchSizeDefault
+	if cfg.Restore.BatchSize > 0 {
+		batchSize = cfg.Restore.BatchSize
+	}
+	numInsertionWorkers := numInsertionWorkersDefault
+	if cfg.Restore.NumInsertionWorkers > 0 {
+		numInsertionWorkers = cfg.Restore.NumInsertionWorkers
+	}
+
+	mopts := mongorestore.Options{}
+	mopts.ToolOptions = topts
+	mopts.InputOptions = &mongorestore.InputOptions{
+		Archive: "-",
+	}
+	mopts.OutputOptions = &mongorestore.OutputOptions{
+		BulkBufferSize:           batchSize,
+		BypassDocumentValidation: true,
+		Drop:                     true,
+		NumInsertionWorkers:      numInsertionWorkers,
+		NumParallelCollections:   1,
+		PreserveUUID:             preserveUUID,
+		StopOnError:              true,
+		WriteConcern:             "majority",
+	}
+	mopts.NSOptions = &mongorestore.NSOptions{
+		NSExclude: excludeFromRestore,
+	}
+
+	mr, err := mongorestore.New(mopts)
+	if err != nil {
+		return errors.Wrap(err, "create mongorestore obj")
+	}
+	mr.SkipUsersAndRoles = true
+	mr.InputReader = input
+
+	rdumpResult := mr.Restore()
+	mr.Close()
+	if rdumpResult.Err != nil {
+		return errors.Wrapf(rdumpResult.Err, "restore mongo dump (successes: %d / fails: %d)", rdumpResult.Successes, rdumpResult.Failures)
+	}
+
+	return nil
+}
+
 // RestoreChunks replays PITR oplog chunks
 func (r *Restore) RestoreChunks() error {
 	r.log.Info("replay chunks")
